@@ -117,6 +117,13 @@ gdk_wayland_keymap_get_num_lock_state (GdkKeymap *keymap)
                                        XKB_LED_NAME_NUM);
 }
 
+static gboolean
+gdk_wayland_keymap_get_scroll_lock_state (GdkKeymap *keymap)
+{
+  return xkb_state_led_name_is_active (GDK_WAYLAND_KEYMAP (keymap)->xkb_state,
+                                       XKB_LED_NAME_SCROLL);
+}
+
 static gboolean
 gdk_wayland_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
 					   guint          keyval,
@@ -373,6 +380,7 @@ _gdk_wayland_keymap_class_init (GdkWaylandKeymapClass *klass)
   keymap_class->have_bidi_layouts = gdk_wayland_keymap_have_bidi_layouts;
   keymap_class->get_caps_lock_state = gdk_wayland_keymap_get_caps_lock_state;
   keymap_class->get_num_lock_state = gdk_wayland_keymap_get_num_lock_state;
+  keymap_class->get_scroll_lock_state = gdk_wayland_keymap_get_scroll_lock_state;
   keymap_class->get_entries_for_keyval = gdk_wayland_keymap_get_entries_for_keyval;
   keymap_class->get_entries_for_keycode = gdk_wayland_keymap_get_entries_for_keycode;
   keymap_class->lookup_key = gdk_wayland_keymap_lookup_key;
