@@ -424,8 +424,7 @@ file_chooser_widget_selection_changed (GtkWidget            *widget,
   sensitive = (uris != NULL);
   gtk_widget_set_sensitive (button, sensitive);
 
-  if (uris)
-    g_slist_free_full (uris, g_free);
+  g_slist_free_full (uris, g_free);
 }
 
 static void
@@ -567,9 +566,11 @@ setup_save_entry (GtkFileChooserDialog *dialog)
       GtkWidget *label;
       GtkWidget *entry;
 
-      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
       label = gtk_label_new_with_mnemonic (_("_Name"));
       entry = _gtk_file_chooser_entry_new (FALSE, FALSE);
+      g_object_set (label, "margin-start", 6, "margin-end", 6, NULL);
+      g_object_set (entry, "margin-start", 6, "margin-end", 6, NULL);
       gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
       gtk_container_add (GTK_CONTAINER (box), label);
       gtk_container_add (GTK_CONTAINER (box), entry);
