@@ -19,6 +19,7 @@
 
 #include "gtkdebug.h"
 #include "gtkpixelcacheprivate.h"
+#include "gtkstylecontextprivate.h"
 
 #define BLOW_CACHE_TIMEOUT_SEC 20
 
@@ -45,6 +46,9 @@ struct _GtkPixelCache {
   /* may be null if not dirty */
   cairo_region_t *surface_dirty;
 
+  /* background tracking for rgb/rgba */
+  GtkStyleContext *style_context;
+
   guint timeout_tag;
 
   guint extra_width;
@@ -88,6 +92,8 @@ _gtk_pixel_cache_free (GtkPixelCache *cache)
   if (cache->surface_dirty != NULL)
     cairo_region_destroy (cache->surface_dirty);
 
+  g_clear_object (&cache->style_context);
+
   g_free (cache);
 }
 
@@ -181,11 +187,9 @@ _gtk_pixel_cache_create_surface_if_needed (GtkPixelCache         *cache,
   cairo_rectangle_int_t rect;
   int surface_w, surface_h;
   cairo_content_t content;
-  cairo_pattern_t *bg;
-  double red, green, blue, alpha;
 
 #ifdef G_ENABLE_DEBUG
-  if (gtk_get_debug_flags () & GTK_DEBUG_NO_PIXEL_CACHE)
+  if (GTK_DEBUG_CHECK (NO_PIXEL_CACHE))
     return;
 #endif
 
@@ -193,11 +197,8 @@ _gtk_pixel_cache_create_surface_if_needed (GtkPixelCache         *cache,
   if (!content)
     {
       content = CAIRO_CONTENT_COLOR_ALPHA;
-      bg = gdk_window_get_background_pattern (window);
-      if (bg != NULL &&
-          cairo_pattern_get_type (bg) == CAIRO_PATTERN_TYPE_SOLID &&
-          cairo_pattern_get_rgba (bg, &red, &green, &blue, &alpha) == CAIRO_STATUS_SUCCESS &&
-          alpha == 1.0)
+      if (cache->style_context &&
+          _gtk_style_context_is_background_opaque (cache->style_context))
         content = CAIRO_CONTENT_COLOR;
     }
 
@@ -363,7 +364,7 @@ _gtk_pixel_cache_repaint (GtkPixelCache         *cache,
       cairo_restore (backing_cr);
 
 #ifdef G_ENABLE_DEBUG
-      if (gtk_get_debug_flags () & GTK_DEBUG_PIXEL_CACHE)
+      if (GTK_DEBUG_CHECK (PIXEL_CACHE))
         {
           GdkRGBA colors[] = {
             { 1, 0, 0, 0.08},
@@ -500,3 +501,11 @@ _gtk_pixel_cache_set_always_cache (GtkPixelCache *cache,
 {
   cache->always_cache = !!always_cache;
 }
+
+void
+_gtk_pixel_cache_set_style_context (GtkPixelCache   *cache,
+                                    GtkStyleContext *style_context)
+{
+  if (g_set_object (&cache->style_context, style_context))
+    _gtk_pixel_cache_invalidate (cache, NULL);
+}
