@@ -22,6 +22,7 @@
 #define __GTK_PLACES_SIDEBAR_PRIVATE_H__
 
 #include <glib.h>
+#include "gtkplacessidebar.h"
 
 G_BEGIN_DECLS
 
@@ -29,9 +30,9 @@ G_BEGIN_DECLS
 typedef enum {
   SECTION_INVALID,
   SECTION_COMPUTER,
-  SECTION_DEVICES,
-  SECTION_NETWORK,
+  SECTION_MOUNTS,
   SECTION_BOOKMARKS,
+  SECTION_OTHER_LOCATIONS,
   N_SECTIONS
 } GtkPlacesSidebarSectionType;
 
@@ -46,9 +47,12 @@ typedef enum {
   PLACES_ENTER_LOCATION,
   PLACES_DROP_FEEDBACK,
   PLACES_BOOKMARK_PLACEHOLDER,
+  PLACES_OTHER_LOCATIONS,
   N_PLACES
 } GtkPlacesSidebarPlaceType;
 
+gchar *gtk_places_sidebar_get_location_title (GtkPlacesSidebar *sidebar);
+
 G_END_DECLS
 
 #endif /* __GTK_PLACES_SIDEBAR_PRIVATE_H__ */
