@@ -260,6 +260,7 @@ static void
 gtk_viewport_init (GtkViewport *viewport)
 {
   GtkViewportPrivate *priv;
+  GtkStyleContext *style_context;
 
   viewport->priv = gtk_viewport_get_instance_private (viewport);
   priv = viewport->priv;
@@ -276,6 +277,9 @@ gtk_viewport_init (GtkViewport *viewport)
 
   priv->pixel_cache = _gtk_pixel_cache_new ();
 
+  style_context = gtk_widget_get_style_context (GTK_WIDGET (viewport));
+  _gtk_pixel_cache_set_style_context (priv->pixel_cache, style_context);
+
   gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (viewport)), GTK_STYLE_CLASS_FRAME);
   viewport_set_adjustment (viewport, GTK_ORIENTATION_HORIZONTAL, NULL);
   viewport_set_adjustment (viewport, GTK_ORIENTATION_VERTICAL, NULL);
@@ -743,7 +747,7 @@ gtk_viewport_realize (GtkWidget *widget)
   attributes.wclass = GDK_INPUT_OUTPUT;
   attributes.visual = gtk_widget_get_visual (widget);
 
-  event_mask = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
+  event_mask = gtk_widget_get_events (widget);
 
   attributes.event_mask = event_mask | GDK_SCROLL_MASK | GDK_TOUCH_MASK | GDK_SMOOTH_SCROLL_MASK;
 
