@@ -17,12 +17,15 @@
 
 #include "config.h"
 
+#include "gtkwidgetpath.h"
+
 #include <string.h>
 
-#include "gtkwidget.h"
-#include "gtkwidgetpath.h"
+#include "gtkcssnodedeclarationprivate.h"
+#include "gtkprivate.h"
 #include "gtkstylecontextprivate.h"
 #include "gtktypebuiltins.h"
+#include "gtkwidget.h"
 
 /**
  * SECTION:gtkwidgetpath
@@ -87,18 +90,14 @@ typedef struct GtkPathElement GtkPathElement;
 
 struct GtkPathElement
 {
-  GType type;
-  GQuark name;
-  GtkStateFlags state;
+  GtkCssNodeDeclaration *decl;
   guint sibling_index;
-  GHashTable *regions;
-  GArray *classes;
   GtkWidgetPath *siblings;
 };
 
 struct _GtkWidgetPath
 {
-  volatile guint ref_count;
+  guint ref_count;
 
   GArray *elems; /* First element contains the described widget */
 };
@@ -130,30 +129,10 @@ gtk_path_element_copy (GtkPathElement       *dest,
 {
   memset (dest, 0, sizeof (GtkPathElement));
 
-  dest->type = src->type;
-  dest->name = src->name;
-  dest->state = src->state;
+  dest->decl = gtk_css_node_declaration_ref (src->decl);
   if (src->siblings)
     dest->siblings = gtk_widget_path_ref (src->siblings);
   dest->sibling_index = src->sibling_index;
-
-  if (src->regions)
-    {
-      GHashTableIter iter;
-      gpointer key, value;
-
-      g_hash_table_iter_init (&iter, src->regions);
-      dest->regions = g_hash_table_new (NULL, NULL);
-
-      while (g_hash_table_iter_next (&iter, &key, &value))
-        g_hash_table_insert (dest->regions, key, value);
-    }
-
-  if (src->classes)
-    {
-      dest->classes = g_array_new (FALSE, FALSE, sizeof (GQuark));
-      g_array_append_vals (dest->classes, src->classes->data, src->classes->len);
-    }
 }
 
 /**
@@ -172,7 +151,7 @@ gtk_widget_path_copy (const GtkWidgetPath *path)
   GtkWidgetPath *new_path;
   guint i;
 
-  g_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
 
   new_path = gtk_widget_path_new ();
 
@@ -204,9 +183,9 @@ gtk_widget_path_copy (const GtkWidgetPath *path)
 GtkWidgetPath *
 gtk_widget_path_ref (GtkWidgetPath *path)
 {
-  g_return_val_if_fail (path != NULL, path);
+  gtk_internal_return_val_if_fail (path != NULL, path);
 
-  g_atomic_int_add (&path->ref_count, 1);
+  path->ref_count += 1;
 
   return path;
 }
@@ -225,9 +204,10 @@ gtk_widget_path_unref (GtkWidgetPath *path)
 {
   guint i;
 
-  g_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path != NULL);
 
-  if (!g_atomic_int_dec_and_test (&path->ref_count))
+  path->ref_count -= 1;
+  if (path->ref_count > 0)
     return;
 
   for (i = 0; i < path->elems->len; i++)
@@ -236,14 +216,7 @@ gtk_widget_path_unref (GtkWidgetPath *path)
 
       elem = &g_array_index (path->elems, GtkPathElement, i);
 
-      if (elem->regions)
-        g_hash_table_destroy (elem->regions);
-
-      if (elem->classes)
-        g_array_free (elem->classes, TRUE);
-
-      if (elem->siblings)
-        gtk_widget_path_unref (elem->siblings);
+      gtk_css_node_declaration_unref (elem->decl);
     }
 
   g_array_free (path->elems, TRUE);
@@ -262,7 +235,7 @@ gtk_widget_path_unref (GtkWidgetPath *path)
 void
 gtk_widget_path_free (GtkWidgetPath *path)
 {
-  g_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path != NULL);
 
   gtk_widget_path_unref (path);
 }
@@ -281,7 +254,7 @@ gtk_widget_path_free (GtkWidgetPath *path)
 gint
 gtk_widget_path_length (const GtkWidgetPath *path)
 {
-  g_return_val_if_fail (path != NULL, 0);
+  gtk_internal_return_val_if_fail (path != NULL, 0);
 
   return path->elems->len;
 }
@@ -305,38 +278,42 @@ char *
 gtk_widget_path_to_string (const GtkWidgetPath *path)
 {
   GString *string;
-  guint i, j;
+  guint i, j, n;
 
-  g_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
 
   string = g_string_new ("");
 
   for (i = 0; i < path->elems->len; i++)
     {
       GtkPathElement *elem;
+      GtkStateFlags state;
+      const GQuark *classes;
+      GList *list, *regions;
 
       elem = &g_array_index (path->elems, GtkPathElement, i);
 
       if (i > 0)
         g_string_append_c (string, ' ');
 
-      g_string_append (string, g_type_name (elem->type));
+      g_string_append (string, g_type_name (gtk_css_node_declaration_get_type (elem->decl)));
 
-      if (elem->name)
+      if (gtk_css_node_declaration_get_id (elem->decl))
         {
           g_string_append_c (string, '(');
-          g_string_append (string, g_quark_to_string (elem->name));
+          g_string_append (string, gtk_css_node_declaration_get_id (elem->decl));
           g_string_append_c (string, ')');
         }
 
-      if (elem->state)
+      state = gtk_css_node_declaration_get_state (elem->decl);
+      if (state)
         {
           GFlagsClass *fclass;
 
           fclass = g_type_class_ref (GTK_TYPE_STATE_FLAGS);
           for (j = 0; j < fclass->n_values; j++)
             {
-              if (elem->state & fclass->values[j].value)
+              if (state & fclass->values[j].value)
                 {
                   g_string_append_c (string, ':');
                   g_string_append (string, fclass->values[j].value_nick);
@@ -350,45 +327,40 @@ gtk_widget_path_to_string (const GtkWidgetPath *path)
                                 elem->sibling_index + 1,
                                 gtk_widget_path_length (elem->siblings));
 
-      if (elem->classes)
+      classes = gtk_css_node_declaration_get_classes (elem->decl, &n);
+      for (j = 0; j < n; j++)
         {
-          for (j = 0; j < elem->classes->len; j++)
-            {
-              g_string_append_c (string, '.');
-              g_string_append (string, g_quark_to_string (g_array_index (elem->classes, GQuark, j)));
-            }
+          g_string_append_c (string, '.');
+          g_string_append (string, g_quark_to_string (classes[j]));
         }
 
-      if (elem->regions)
+      regions = gtk_css_node_declaration_list_regions (elem->decl);
+      for (list = regions; list; list = list->next)
         {
-          GHashTableIter iter;
-          gpointer key, value;
+          static const char *flag_names[] = {
+            "even",
+            "odd",
+            "first",
+            "last",
+            "only",
+            "sorted"
+          };
+          GtkRegionFlags flags;
+          GQuark region = GPOINTER_TO_UINT (regions->data);
 
-          g_hash_table_iter_init (&iter, elem->regions);
-          while (g_hash_table_iter_next (&iter, &key, &value))
+          gtk_css_node_declaration_has_region (elem->decl, region, &flags);
+          g_string_append_c (string, ' ');
+          g_string_append (string, g_quark_to_string (region));
+          for (j = 0; j < G_N_ELEMENTS(flag_names); j++)
             {
-              GtkRegionFlags flags = GPOINTER_TO_UINT (value);
-              static const char *flag_names[] = {
-                "even",
-                "odd",
-                "first",
-                "last",
-                "only",
-                "sorted"
-              };
-
-              g_string_append_c (string, ' ');
-              g_string_append (string, g_quark_to_string (GPOINTER_TO_UINT (key)));
-              for (j = 0; j < G_N_ELEMENTS(flag_names); j++)
+              if (flags & (1 << j))
                 {
-                  if (flags & (1 << j))
-                    {
-                      g_string_append_c (string, ':');
-                      g_string_append (string, flag_names[j]);
-                    }
+                  g_string_append_c (string, ':');
+                  g_string_append (string, flag_names[j]);
                 }
             }
         }
+      g_list_free (regions);
     }
 
   return g_string_free (string, FALSE);
@@ -407,11 +379,13 @@ void
 gtk_widget_path_prepend_type (GtkWidgetPath *path,
                               GType          type)
 {
-  GtkPathElement new = { 0 };
+  GtkPathElement new = { NULL };
 
-  g_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path != NULL);
+
+  new.decl = gtk_css_node_declaration_new ();
+  gtk_css_node_declaration_set_type (&new.decl, type);
 
-  new.type = type;
   g_array_prepend_val (path->elems, new);
 }
 
@@ -430,11 +404,12 @@ gint
 gtk_widget_path_append_type (GtkWidgetPath *path,
                              GType          type)
 {
-  GtkPathElement new = { 0 };
+  GtkPathElement new = { NULL };
 
-  g_return_val_if_fail (path != NULL, 0);
+  gtk_internal_return_val_if_fail (path != NULL, 0);
 
-  new.type = type;
+  new.decl = gtk_css_node_declaration_new ();
+  gtk_css_node_declaration_set_type (&new.decl, type);
   g_array_append_val (path->elems, new);
 
   return path->elems->len - 1;
@@ -468,9 +443,9 @@ gtk_widget_path_append_with_siblings (GtkWidgetPath *path,
 {
   GtkPathElement new;
 
-  g_return_val_if_fail (path != NULL, 0);
-  g_return_val_if_fail (siblings != NULL, 0);
-  g_return_val_if_fail (sibling_index < gtk_widget_path_length (siblings), 0);
+  gtk_internal_return_val_if_fail (path != NULL, 0);
+  gtk_internal_return_val_if_fail (siblings != NULL, 0);
+  gtk_internal_return_val_if_fail (sibling_index < gtk_widget_path_length (siblings), 0);
 
   gtk_path_element_copy (&new, &g_array_index (siblings->elems, GtkPathElement, sibling_index));
   new.siblings = gtk_widget_path_ref (siblings);
@@ -496,8 +471,8 @@ gtk_widget_path_iter_get_siblings (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, NULL);
-  g_return_val_if_fail (path->elems->len != 0, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
@@ -524,8 +499,8 @@ gtk_widget_path_iter_get_sibling_index (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, G_TYPE_INVALID);
-  g_return_val_if_fail (path->elems->len != 0, G_TYPE_INVALID);
+  gtk_internal_return_val_if_fail (path != NULL, G_TYPE_INVALID);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, G_TYPE_INVALID);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
@@ -552,14 +527,14 @@ gtk_widget_path_iter_get_object_type (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, G_TYPE_INVALID);
-  g_return_val_if_fail (path->elems->len != 0, G_TYPE_INVALID);
+  gtk_internal_return_val_if_fail (path != NULL, G_TYPE_INVALID);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, G_TYPE_INVALID);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
-  return elem->type;
+  return gtk_css_node_declaration_get_type (elem->decl);
 }
 
 /**
@@ -580,14 +555,14 @@ gtk_widget_path_iter_set_object_type (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
-  elem->type = type;
+  gtk_css_node_declaration_set_type (&elem->decl, type);
 }
 
 /**
@@ -609,14 +584,14 @@ gtk_widget_path_iter_get_state (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, 0);
-  g_return_val_if_fail (path->elems->len != 0, 0);
+  gtk_internal_return_val_if_fail (path != NULL, 0);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, 0);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
-  return elem->state;
+  return gtk_css_node_declaration_get_state (elem->decl);
 }
 
 /**
@@ -653,15 +628,15 @@ gtk_widget_path_iter_set_state (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  elem->state = state;
+  gtk_css_node_declaration_set_state (&elem->decl, state);
 }
 
 /**
@@ -681,14 +656,14 @@ gtk_widget_path_iter_get_name (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, NULL);
-  g_return_val_if_fail (path->elems->len != 0, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
-  return g_quark_to_string (elem->name);
+  return gtk_css_node_declaration_get_id (elem->decl);
 }
 
 /**
@@ -709,16 +684,16 @@ gtk_widget_path_iter_set_name (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
-  g_return_if_fail (name != NULL);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (name != NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  elem->name = g_quark_from_string (name);
+  gtk_css_node_declaration_set_id (&elem->decl, g_intern_string (name));
 }
 
 /**
@@ -739,18 +714,11 @@ gtk_widget_path_iter_has_qname (const GtkWidgetPath *path,
                                 gint                 pos,
                                 GQuark               qname)
 {
-  GtkPathElement *elem;
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (qname != 0, FALSE);
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
-  g_return_val_if_fail (qname != 0, FALSE);
-
-  if (pos < 0 || pos >= path->elems->len)
-    pos = path->elems->len - 1;
-
-  elem = &g_array_index (path->elems, GtkPathElement, pos);
-
-  return (elem->name == qname);
+  return gtk_widget_path_iter_has_name (path, pos, g_quark_to_string (qname));
 }
 
 /**
@@ -771,20 +739,18 @@ gtk_widget_path_iter_has_name (const GtkWidgetPath *path,
                                gint                 pos,
                                const gchar         *name)
 {
-  GQuark qname;
+  GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
-  qname = g_quark_try_string (name);
+  name = g_intern_string (name);
+  elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (qname == 0)
-    return FALSE;
-
-  return gtk_widget_path_iter_has_qname (path, pos, qname);
+  return gtk_css_node_declaration_get_id (elem->decl) == name;
 }
 
 /**
@@ -804,46 +770,26 @@ gtk_widget_path_iter_add_class (GtkWidgetPath *path,
                                 gint           pos,
                                 const gchar   *name)
 {
-  GtkPathElement *elem;
-  gboolean added = FALSE;
-  GQuark qname;
-  guint i;
-
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
-  g_return_if_fail (name != NULL);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (name != NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
+  gtk_widget_path_iter_add_qclass (path, pos, g_quark_from_string (name));
+}
+
+void
+gtk_widget_path_iter_add_qclass (GtkWidgetPath *path,
+                                 gint           pos,
+                                 GQuark         qname)
+{
+  GtkPathElement *elem;
+
   elem = &g_array_index (path->elems, GtkPathElement, pos);
-  qname = g_quark_from_string (name);
 
-  if (!elem->classes)
-    elem->classes = g_array_new (FALSE, FALSE, sizeof (GQuark));
-
-  for (i = 0; i < elem->classes->len; i++)
-    {
-      GQuark quark;
-
-      quark = g_array_index (elem->classes, GQuark, i);
-
-      if (qname == quark)
-        {
-          /* Already there */
-          added = TRUE;
-          break;
-        }
-      if (qname < quark)
-        {
-          g_array_insert_val (elem->classes, i, qname);
-          added = TRUE;
-          break;
-        }
-    }
-
-  if (!added)
-    g_array_append_val (elem->classes, qname);
+  gtk_css_node_declaration_add_class (&elem->decl, qname);
 }
 
 /**
@@ -864,39 +810,20 @@ gtk_widget_path_iter_remove_class (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
   GQuark qname;
-  guint i;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
-  g_return_if_fail (name != NULL);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (name != NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
+  elem = &g_array_index (path->elems, GtkPathElement, pos);
   qname = g_quark_try_string (name);
-
   if (qname == 0)
     return;
 
-  elem = &g_array_index (path->elems, GtkPathElement, pos);
-
-  if (!elem->classes)
-    return;
-
-  for (i = 0; i < elem->classes->len; i++)
-    {
-      GQuark quark;
-
-      quark = g_array_index (elem->classes, GQuark, i);
-
-      if (quark > qname)
-        break;
-      else if (quark == qname)
-        {
-          g_array_remove_index (elem->classes, i);
-          break;
-        }
-    }
+  gtk_css_node_declaration_remove_class (&elem->decl, qname);
 }
 
 /**
@@ -915,19 +842,15 @@ gtk_widget_path_iter_clear_classes (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (!elem->classes)
-    return;
-
-  if (elem->classes->len > 0)
-    g_array_remove_range (elem->classes, 0, elem->classes->len);
+  gtk_css_node_declaration_clear_classes (&elem->decl);
 }
 
 /**
@@ -951,25 +874,21 @@ gtk_widget_path_iter_list_classes (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
   GSList *list = NULL;
-  guint i;
+  const GQuark *classes;
+  guint i, n;
 
-  g_return_val_if_fail (path != NULL, NULL);
-  g_return_val_if_fail (path->elems->len != 0, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
+  classes = gtk_css_node_declaration_get_classes (elem->decl, &n);
 
-  if (!elem->classes)
-    return NULL;
-
-  for (i = 0; i < elem->classes->len; i++)
+  for (i = 0; i < n; i++)
     {
-      GQuark quark;
-
-      quark = g_array_index (elem->classes, GQuark, i);
-      list = g_slist_prepend (list, (gchar *) g_quark_to_string (quark));
+      list = g_slist_prepend (list, (gchar *) g_quark_to_string (classes[i]));
     }
 
   return g_slist_reverse (list);
@@ -994,33 +913,17 @@ gtk_widget_path_iter_has_qclass (const GtkWidgetPath *path,
                                  GQuark               qname)
 {
   GtkPathElement *elem;
-  guint i;
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
-  g_return_val_if_fail (qname != 0, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (qname != 0, FALSE);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (!elem->classes)
-    return FALSE;
-
-  for (i = 0; i < elem->classes->len; i++)
-    {
-      GQuark quark;
-
-      quark = g_array_index (elem->classes, GQuark, i);
-
-      if (quark == qname)
-        return TRUE;
-      else if (quark > qname)
-        break;
-    }
-
-  return FALSE;
+  return gtk_css_node_declaration_has_class (elem->decl, qname);
 }
 
 /**
@@ -1043,9 +946,9 @@ gtk_widget_path_iter_has_class (const GtkWidgetPath *path,
 {
   GQuark qname;
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
-  g_return_val_if_fail (name != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (name != NULL, FALSE);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
@@ -1085,10 +988,10 @@ gtk_widget_path_iter_add_region (GtkWidgetPath  *path,
   GtkPathElement *elem;
   GQuark qname;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
-  g_return_if_fail (name != NULL);
-  g_return_if_fail (_gtk_style_context_check_region_name (name));
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (name != NULL);
+  gtk_internal_return_if_fail (_gtk_style_context_check_region_name (name));
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
@@ -1096,12 +999,7 @@ gtk_widget_path_iter_add_region (GtkWidgetPath  *path,
   elem = &g_array_index (path->elems, GtkPathElement, pos);
   qname = g_quark_from_string (name);
 
-  if (!elem->regions)
-    elem->regions = g_hash_table_new (NULL, NULL);
-
-  g_hash_table_insert (elem->regions,
-                       GUINT_TO_POINTER (qname),
-                       GUINT_TO_POINTER (flags));
+  gtk_css_node_declaration_add_region (&elem->decl, qname, flags);
 }
 
 /**
@@ -1125,22 +1023,19 @@ gtk_widget_path_iter_remove_region (GtkWidgetPath *path,
   GtkPathElement *elem;
   GQuark qname;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
-  g_return_if_fail (name != NULL);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (name != NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
+  elem = &g_array_index (path->elems, GtkPathElement, pos);
   qname = g_quark_try_string (name);
-
   if (qname == 0)
     return;
 
-  elem = &g_array_index (path->elems, GtkPathElement, pos);
-
-  if (elem->regions)
-    g_hash_table_remove (elem->regions, GUINT_TO_POINTER (qname));
+  gtk_css_node_declaration_remove_region (&elem->decl, qname);
 }
 
 /**
@@ -1161,16 +1056,15 @@ gtk_widget_path_iter_clear_regions (GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_if_fail (path != NULL);
-  g_return_if_fail (path->elems->len != 0);
+  gtk_internal_return_if_fail (path != NULL);
+  gtk_internal_return_if_fail (path->elems->len != 0);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (elem->regions)
-    g_hash_table_remove_all (elem->regions);
+  gtk_css_node_declaration_clear_regions (&elem->decl);
 }
 
 /**
@@ -1195,30 +1089,23 @@ gtk_widget_path_iter_list_regions (const GtkWidgetPath *path,
                                    gint                 pos)
 {
   GtkPathElement *elem;
-  GHashTableIter iter;
   GSList *list = NULL;
-  gpointer key;
+  GList *l, *wrong_list;
 
-  g_return_val_if_fail (path != NULL, NULL);
-  g_return_val_if_fail (path->elems->len != 0, NULL);
+  gtk_internal_return_val_if_fail (path != NULL, NULL);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, NULL);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (!elem->regions)
-    return NULL;
-
-  g_hash_table_iter_init (&iter, elem->regions);
-
-  while (g_hash_table_iter_next (&iter, &key, NULL))
+  wrong_list = gtk_css_node_declaration_list_regions (elem->decl);
+  for (l = wrong_list; l; l = l->next)
     {
-      GQuark qname;
-
-      qname = GPOINTER_TO_UINT (key);
-      list = g_slist_prepend (list, (gchar *) g_quark_to_string (qname));
+      list = g_slist_prepend (list, (char *) g_quark_to_string (GPOINTER_TO_UINT (l->data)));
     }
+  g_list_free (wrong_list);
 
   return list;
 }
@@ -1246,29 +1133,17 @@ gtk_widget_path_iter_has_qregion (const GtkWidgetPath *path,
                                   GtkRegionFlags      *flags)
 {
   GtkPathElement *elem;
-  gpointer value;
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
-  g_return_val_if_fail (qname != 0, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (qname != 0, FALSE);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
 
   elem = &g_array_index (path->elems, GtkPathElement, pos);
 
-  if (!elem->regions)
-    return FALSE;
-
-  if (!g_hash_table_lookup_extended (elem->regions,
-                                     GUINT_TO_POINTER (qname),
-                                     NULL, &value))
-    return FALSE;
-
-  if (flags)
-    *flags = GPOINTER_TO_UINT (value);
-
-  return TRUE;
+  return gtk_css_node_declaration_has_region (elem->decl, qname, flags);
 }
 
 /**
@@ -1295,9 +1170,9 @@ gtk_widget_path_iter_has_region (const GtkWidgetPath *path,
 {
   GQuark qname;
 
-  g_return_val_if_fail (path != NULL, FALSE);
-  g_return_val_if_fail (path->elems->len != 0, FALSE);
-  g_return_val_if_fail (name != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path->elems->len != 0, FALSE);
+  gtk_internal_return_val_if_fail (name != NULL, FALSE);
 
   if (pos < 0 || pos >= path->elems->len)
     pos = path->elems->len - 1;
@@ -1328,11 +1203,11 @@ gtk_widget_path_get_object_type (const GtkWidgetPath *path)
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, G_TYPE_INVALID);
+  gtk_internal_return_val_if_fail (path != NULL, G_TYPE_INVALID);
 
   elem = &g_array_index (path->elems, GtkPathElement,
                          path->elems->len - 1);
-  return elem->type;
+  return gtk_css_node_declaration_get_type (elem->decl);
 }
 
 /**
@@ -1353,16 +1228,12 @@ gtk_widget_path_is_type (const GtkWidgetPath *path,
 {
   GtkPathElement *elem;
 
-  g_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
 
   elem = &g_array_index (path->elems, GtkPathElement,
                          path->elems->len - 1);
 
-  if (elem->type == type ||
-      g_type_is_a (elem->type, type))
-    return TRUE;
-
-  return FALSE;
+  return g_type_is_a (gtk_css_node_declaration_get_type (elem->decl), type);
 }
 
 /**
@@ -1383,7 +1254,7 @@ gtk_widget_path_has_parent (const GtkWidgetPath *path,
 {
   guint i;
 
-  g_return_val_if_fail (path != NULL, FALSE);
+  gtk_internal_return_val_if_fail (path != NULL, FALSE);
 
   for (i = 0; i < path->elems->len - 1; i++)
     {
@@ -1391,8 +1262,7 @@ gtk_widget_path_has_parent (const GtkWidgetPath *path,
 
       elem = &g_array_index (path->elems, GtkPathElement, i);
 
-      if (elem->type == type ||
-          g_type_is_a (elem->type, type))
+      if (g_type_is_a (gtk_css_node_declaration_get_type (elem->decl), type))
         return TRUE;
     }
 
