@@ -27,11 +27,129 @@
 
 #include "gtkcsstypesprivate.h"
 #include "gtkwidget.h"
+#include "gtkcontainer.h"
 #include "gtkeventcontroller.h"
 #include "gtkactionmuxer.h"
+#include "gtksizerequestcacheprivate.h"
 
 G_BEGIN_DECLS
 
+#define GTK_STATE_FLAGS_BITS 12
+
+struct _GtkWidgetPrivate
+{
+  /* The state of the widget. Needs to be able to hold all GtkStateFlags bits
+   * (defined in "gtkenums.h").
+   */
+  guint state_flags : GTK_STATE_FLAGS_BITS;
+
+  guint direction             : 2;
+
+  guint in_destruction        : 1;
+  guint toplevel              : 1;
+  guint anchored              : 1;
+  guint composite_child       : 1;
+  guint no_window             : 1;
+  guint realized              : 1;
+  guint mapped                : 1;
+  guint visible               : 1;
+  guint sensitive             : 1;
+  guint can_focus             : 1;
+  guint has_focus             : 1;
+  guint can_default           : 1;
+  guint has_default           : 1;
+  guint receives_default      : 1;
+  guint has_grab              : 1;
+  guint shadowed              : 1;
+  guint style_update_pending  : 1;
+  guint app_paintable         : 1;
+  guint double_buffered       : 1;
+  guint redraw_on_alloc       : 1;
+  guint no_show_all           : 1;
+  guint child_visible         : 1;
+  guint multidevice           : 1;
+  guint has_shape_mask        : 1;
+  guint in_reparent           : 1;
+
+  /* Queue-resize related flags */
+  guint alloc_needed          : 1;
+
+  /* Expand-related flags */
+  guint need_compute_expand   : 1; /* Need to recompute computed_[hv]_expand */
+  guint computed_hexpand      : 1; /* computed results (composite of child flags) */
+  guint computed_vexpand      : 1;
+  guint hexpand               : 1; /* application-forced expand */
+  guint vexpand               : 1;
+  guint hexpand_set           : 1; /* whether to use application-forced  */
+  guint vexpand_set           : 1; /* instead of computing from children */
+
+  /* SizeGroup related flags */
+  guint have_size_groups      : 1;
+
+  /* Alignment */
+  guint   halign              : 4;
+  guint   valign              : 4;
+
+  guint8 alpha;
+  guint8 user_alpha;
+
+#ifdef G_ENABLE_CONSISTENCY_CHECKS
+  /* Number of gtk_widget_push_verify_invariants () */
+  guint8 verifying_invariants_count;
+#endif
+
+  gint width;
+  gint height;
+  GtkBorder margin;
+
+  /* Animations and other things to update on clock ticks */
+  guint clock_tick_id;
+  GList *tick_callbacks;
+
+  /* The widget's name. If the widget does not have a name
+   * (the name is NULL), then its name (as returned by
+   * "gtk_widget_get_name") is its class's name.
+   * Among other things, the widget name is used to determine
+   * the style to use for a widget.
+   */
+  gchar *name;
+
+  /* The list of attached windows to this widget.
+   * We keep a list in order to call reset_style to all of them,
+   * recursively.
+   */
+  GList *attached_windows;
+
+  /* The style for the widget. The style contains the
+   * colors the widget should be drawn in for each state
+   * along with graphics contexts used to draw with and
+   * the font to use for text.
+   */
+  GtkStyle *style;
+  GtkCssNode *cssnode;
+  GtkStyleContext *context;
+
+  /* The widget's allocated size */
+  GtkAllocation allocation;
+  GtkAllocation clip;
+  gint allocated_baseline;
+
+  /* The widget's requested sizes */
+  SizeRequestCache requests;
+
+  /* The widget's window or its parent window if it does
+   * not have a window. (Which will be indicated by the
+   * no_window field being set).
+   */
+  GdkWindow *window;
+  GList *registered_windows;
+
+  /* The widget's parent */
+  GtkWidget *parent;
+
+  GList *event_controllers;
+};
+
 GtkCssNode *  gtk_widget_get_css_node       (GtkWidget *widget);
 void         _gtk_widget_set_visible_flag   (GtkWidget *widget,
                                              gboolean   visible);
@@ -126,7 +244,7 @@ void              _gtk_widget_synthesize_crossing          (GtkWidget       *fro
                                                             GdkDevice       *device,
                                                             GdkCrossingMode  mode);
 
-gpointer          _gtk_widget_peek_request_cache           (GtkWidget *widget);
+static inline gpointer _gtk_widget_peek_request_cache           (GtkWidget *widget);
 
 void              _gtk_widget_buildable_finish_accelerator (GtkWidget *widget,
                                                             GtkWidget *toplevel,
@@ -173,6 +291,85 @@ void              gtk_widget_set_csd_input_shape           (GtkWidget
 
 gboolean          gtk_widget_has_size_request              (GtkWidget *widget);
 
+/* inline getters */
+
+static inline GtkWidget *
+_gtk_widget_get_parent (GtkWidget *widget)
+{
+  return widget->priv->parent;
+}
+
+static inline gboolean
+_gtk_widget_get_visible (GtkWidget *widget)
+{
+  return widget->priv->visible;
+}
+
+static inline gboolean
+_gtk_widget_get_child_visible (GtkWidget *widget)
+{
+  return widget->priv->child_visible;
+}
+
+static inline gboolean
+_gtk_widget_get_mapped (GtkWidget *widget)
+{
+  return widget->priv->mapped;
+}
+
+static inline gboolean
+_gtk_widget_get_realized (GtkWidget *widget)
+{
+  return widget->priv->realized;
+}
+
+static inline gboolean
+_gtk_widget_is_toplevel (GtkWidget *widget)
+{
+  return widget->priv->toplevel;
+}
+
+static inline GtkStateFlags
+_gtk_widget_get_state_flags (GtkWidget *widget)
+{
+  return widget->priv->state_flags;
+}
+
+extern GtkTextDirection gtk_default_direction;
+
+static inline GtkTextDirection
+_gtk_widget_get_direction (GtkWidget *widget)
+{
+  if (widget->priv->direction == GTK_TEXT_DIR_NONE)
+    return gtk_default_direction;
+  else
+    return widget->priv->direction;
+}
+
+static inline GtkWidget *
+_gtk_widget_get_toplevel (GtkWidget *widget)
+{
+  while (widget->priv->parent)
+    widget = widget->priv->parent;
+
+  return widget;
+}
+
+static inline GtkStyleContext *
+_gtk_widget_get_style_context (GtkWidget *widget)
+{
+  if (G_LIKELY (widget->priv->context))
+    return widget->priv->context;
+
+  return gtk_widget_get_style_context (widget);
+}
+
+static inline gpointer
+_gtk_widget_peek_request_cache (GtkWidget *widget)
+{
+  return &widget->priv->requests;
+}
+
 G_END_DECLS
 
 #endif /* __GTK_WIDGET_PRIVATE_H__ */
